#include "pins_arduino.h"
#include "stdio.h"
#include "avr/pgmspace.h"
#include "Accel2Spi.h"


void SPIbegin() {

  // Set SS to high so a connected chip will be "deselected" by default
  digitalWrite(SS, HIGH);

  // When the SS pin is set as OUTPUT, it can be used as
  // a general purpose output port (it doesn't influence
  // SPI operations).
  pinMode(SS, OUTPUT);

  // Warning: if the SS pin ever becomes a LOW INPUT then SPI
  // automatically switches to Slave, so the data direction of
  // the SS pin MUST be kept as OUTPUT.
  SPCR |= _BV(MSTR);
  SPCR |= _BV(SPE);

  // Set direction register for SCK and MOSI pin.
  // MISO pin automatically overrides to INPUT.
  // By doing this AFTER enabling SPI, we avoid accidentally
  // clocking in a single bit since the lines go directly
  // from "input" to SPI control.  
  // http://code.google.com/p/arduino/issues/detail?id=888
  pinMode(SCK, OUTPUT);
  pinMode(MOSI, OUTPUT);
}


void SPIend() {
  SPCR &= ~_BV(SPE);
}

void SPIsetBitOrder(uint8_t bitOrder)
{
  if(bitOrder == LSBFIRST) {
    SPCR |= _BV(DORD);
  } else {
    SPCR &= ~(_BV(DORD));
  }
}

void SPIsetDataMode(uint8_t mode)
{
  SPCR = (SPCR & ~SPI_MODE_MASK) | mode;
}

void SPIsetClockDivider(uint8_t rate)
{
  SPCR = (SPCR & ~SPI_CLOCK_MASK) | (rate & SPI_CLOCK_MASK);
  SPSR = (SPSR & ~SPI_2XCLOCK_MASK) | ((rate >> 2) & SPI_2XCLOCK_MASK);
}

byte SPItransfer(byte _data) {
  SPDR = _data;
  while (!(SPSR & _BV(SPIF)))
    ;
  return SPDR;
}
//-----------------------------------------------------------------------
const unsigned int SLAVE_SELECT_PIN = SS; // Slave Select or Chip Select

// MMA7455
const unsigned char SPI_READ = 0x00; // Bit 7 = 0 for reading
const unsigned char SPI_WRITE = 0x80; // Bit 7 = 1 for writing
const unsigned char REG_MCTL = 0x16; // Mode Control

unsigned int inX[100], inY[100], inZ[100];

//setting accelerometer 
void setOffsetValueAccel(unsigned char address,unsigned int value) {
   digitalWrite( SLAVE_SELECT_PIN, LOW );
   byte command = SPI_WRITE | ( address << 1 );
   SPItransfer( command );
   SPItransfer( value%256 );
   SPItransfer(value/256);
   digitalWrite( SLAVE_SELECT_PIN, HIGH );
   delay( 100 );
}

void writeRegisterValueByteAccel(unsigned char address,unsigned char value) {
   digitalWrite( SLAVE_SELECT_PIN, LOW );
   byte command = SPI_WRITE | ( address << 1 );
   SPItransfer( command );
   SPItransfer( value );
   digitalWrite( SLAVE_SELECT_PIN, HIGH );
   delay( 100 );
}

void setModeControl(unsigned char value) {
    writeRegisterValueByteAccel(REG_MCTL,value);   
}

unsigned char readRegisterValueByteAccel(unsigned char address) {   
   unsigned char retValue = 0x00;
   
   digitalWrite( SLAVE_SELECT_PIN, LOW );
   byte command = SPI_READ | ( address << 1 );
   SPItransfer( command );
   retValue = SPItransfer( 0x00 );
   digitalWrite( SLAVE_SELECT_PIN, HIGH );
   
   return retValue;  
}

unsigned int readRegisterValueWordAccel(unsigned char address) {
        unsigned int value,tempValue; 
        
        digitalWrite( SLAVE_SELECT_PIN, LOW );
        byte command = SPI_READ | ( address << 1 );
        SPItransfer( command );
        tempValue = SPItransfer( 0x01 );
        value = SPItransfer( 0x00 );
        digitalWrite( SLAVE_SELECT_PIN, HIGH );
   
        value = value << 8;
        value += tempValue;
        
        return value;
}

void TenBitTwoComplementToSignAndValue(unsigned int input,unsigned char* sign,
unsigned int* value) {
    if ((input & 0x0200)==0x0200) {       
        *sign = '-'; 
        input = ~input;   
        input = input & 0x01FF;
        input = input + 1;        
        *value = input;            
    }               
    else {
        *sign = '+'; 
        *value = input;
    }
}

unsigned int SignAndValueToTwoComplementOffset(unsigned char sign,unsigned int value) {

    if (sign=='-') {       
        value = ~value;
        value = value + 1;
        value = value & 0x7FF;
    }               
    return value;
}

// filter
void movingAverage(int dataX, int dataY, int dataZ)
{
  int inputX, inputY, inputZ;           
    inX[0] = dataX;
    inY[0] = dataY;
    inZ[0] = dataZ;
    //Low pass filter
    inputX = inX[0]*1 + inX[1]*3 + inX[2]*5 + inX[3]*3 + inX[4]*1;
    inputY = inY[0]*1 + inY[1]*3 + inY[2]*5 + inY[3]*3 + inY[4]*1;
    inputZ = inZ[0]*1 + inZ[1]*3 + inZ[2]*5 + inZ[3]*3 + inZ[4]*1;
    dataX = inputX/13;
    dataY = inputY/13;
    dataZ = inputZ/13;
    
    inX[4] = inX[3];
    inX[3] = inX[2];
    inX[2] = inX[1];
    inX[1] = inX[0]; 
    inY[4] = inY[3];
    inY[3] = inY[2];
    inY[2] = inY[1];
    inY[1] = inY[0]; 
    inZ[4] = inZ[3];
    inZ[3] = inZ[2];
    inZ[2] = inZ[1];
    inZ[1] = inZ[0];    
}

void average(int dataX, int dataY, int dataZ)
{
  int inputX, inputY, inputZ;
  const int length = 100;
  
  inX[0] = dataX;
  inY[0] = dataY;
  inZ[0] = dataZ;
    
  for(int i=0; i < length; i++)
  {
    inputX += inX[i];
    inputY += inY[i];
    inputZ += inZ[i];
  }
  
  dataX = inputX/length;
  dataY = inputY/length;
  dataZ = inputZ/length;
  
  for(int i=length-2; i >= 0; i--)
  {
    inX[i+1] = inX[i];
    inY[i+1] = inY[i];
    inZ[i+1] = inZ[i];
  }
}

void SerialAccel1(int signx, int accelx, int signy, int accely, int signz, int accelz)
{
  Serial.write(signx);
  Serial.write(accelx >> 8);
  Serial.write(accelx);
  Serial.write(signy);
  Serial.write(accely >> 8);
  Serial.write(accely);
  Serial.write(signz);
  Serial.write(accelz >> 8);
  Serial.write(accelz);
}

void SerialAccel2(int signx, int accelx, int signy, int accely, int signz, int accelz)
{
  Serial.print(' ');
  Serial.print(char(signx));
  Serial.print(accelx);
  Serial.print(' ');
  Serial.print(char(signy));
  Serial.print(accely);
  Serial.print(' ');
  Serial.print(char(signz));
  Serial.print(accelz);
}

void spiAccel_init()
{
  pinMode(SS, OUTPUT );
  SPIbegin();
  SPIsetBitOrder( MSBFIRST );
  SPIsetDataMode( SPI_MODE0 );
  SPIsetClockDivider( SPI_CLOCK_DIV128 );
}