#include "pins_arduino.h"
#include "stdio.h"
#include "avr/pgmspace.h"
#include "SettingGyroSPI.h"

void SPIbegin() {

  // Set SS to high so a connected chip will be "deselected" by default
  digitalWrite(SS, HIGH);

  // When the SS pin is set as OUTPUT, it can be used as
  // a general purpose output port (it doesn't influence
  // SPI operations).
  pinMode(SS, OUTPUT);

  // Warning: if the SS pin ever becomes a LOW INPUT then SPI
  // automatically switches to Slave, so the data direction of
  // the SS pin MUST be kept as OUTPUT.
  SPCR |= _BV(MSTR);
  SPCR |= _BV(SPE);

  // Set direction register for SCK and MOSI pin.
  // MISO pin automatically overrides to INPUT.
  // By doing this AFTER enabling SPI, we avoid accidentally
  // clocking in a single bit since the lines go directly
  // from "input" to SPI control.  
  // http://code.google.com/p/arduino/issues/detail?id=888
  pinMode(SCK, OUTPUT);
  pinMode(MOSI, OUTPUT);
}


void SPIend() {
  SPCR &= ~_BV(SPE);
}

void SPIsetBitOrder(uint8_t bitOrder)
{
  if(bitOrder == LSBFIRST) {
    SPCR |= _BV(DORD);
  } else {
    SPCR &= ~(_BV(DORD));
  }
}

void SPIsetDataMode(uint8_t mode)
{
  SPCR = (SPCR & ~SPI_MODE_MASK) | mode;
}

void SPIsetClockDivider(uint8_t rate)
{
  SPCR = (SPCR & ~SPI_CLOCK_MASK) | (rate & SPI_CLOCK_MASK);
  SPSR = (SPSR & ~SPI_2XCLOCK_MASK) | ((rate >> 2) & SPI_2XCLOCK_MASK);
}

byte SPItransfer(byte _data) {
  SPDR = _data;
  while (!(SPSR & _BV(SPIF)))
    ;
  return SPDR;
}
//-----------------------------------------------------------------------

const unsigned int SLAVE_SELECT_PIN = SS; // Slave Select or Chip Select

const unsigned char SPI_READ = 0x80; // Bit 7 = 0 for reading
const unsigned char SPI_WRITE = 0x7F; // Bit 7 = 1 for writing

int x_gyro,y_gyro,z_gyro;
unsigned char a,b,c,d,e,f;

// settting gyroscope
void writeRegisterValueByteGyro(unsigned char address,unsigned char value) {
   digitalWrite( SLAVE_SELECT_PIN, LOW );
   byte command = SPI_WRITE & address;
   SPItransfer( command );
   SPItransfer( value );
   digitalWrite( SLAVE_SELECT_PIN, HIGH );
   delay( 100 );
}

unsigned char readRegisterValueByteGyro(unsigned char address, unsigned char ack) {
        unsigned char value;
        
        digitalWrite( SLAVE_SELECT_PIN, LOW );
		byte command = SPI_READ | address;
		SPItransfer( command );
		value = SPItransfer( ack );
		digitalWrite( SLAVE_SELECT_PIN, HIGH );
        
        return value;
} 

void readL3G4200D(void) {

        a = readRegisterValueByteGyro(0x28,0x01);
        b = readRegisterValueByteGyro(0x29,0x01);
        c = readRegisterValueByteGyro(0x2A,0x01);
        d = readRegisterValueByteGyro(0x2B,0x01);
        e = readRegisterValueByteGyro(0x2C,0x01);
        f = readRegisterValueByteGyro(0x2D,0x00);        

        x_gyro = (int)b; 
        x_gyro = x_gyro << 8;   
        x_gyro |= a;    
        x_gyro = x_gyro /128;
        
        y_gyro = (int)d; 
        y_gyro = y_gyro << 8;   
        y_gyro |=c;
        y_gyro = y_gyro /128;
        
        z_gyro = (int)f; 
        z_gyro = z_gyro << 8;   
        z_gyro |= e;
        z_gyro = z_gyro /128;

}

void SerialGyro1()
{
  Serial.write(x_gyro >> 8);
  Serial.write(x_gyro);
  Serial.write(y_gyro >> 8);
  Serial.write(y_gyro);
  Serial.write(z_gyro >> 8);
  Serial.write(z_gyro);
}

void SerialGyro2()
{
  Serial.print(x_gyro);
  Serial.print(' ');
  Serial.print(y_gyro);
  Serial.print(' ');
  Serial.print(z_gyro); 
}

void gyroSpi_init()
{
  pinMode(SS, OUTPUT );
  SPIbegin();
  SPIsetBitOrder( MSBFIRST );
  SPIsetDataMode( SPI_MODE0 );
  SPIsetClockDivider( SPI_CLOCK_DIV128 );
}